from flask import Flask
from webservices.models import Provider
from webservices.sync import provider_for_flask
import os

app = Flask(__name__)


with open(os.path.join(os.path.dirname(__file__), 'keys.txt')) as fobj:
    data = fobj.read()
    app.keys = dict([
        line.split(':')
        for line in data.split('\n')
        if line.strip()
    ])


class HelloProvider(Provider):
    def get_private_key(self, public_key):
        private_key = app.keys.get(public_key)
        return private_key

    def provide(self, data):
        name = data.get('name', 'world')
        return {'greeting': u'hello %s' % name}


provider_for_flask(app, '/', HelloProvider())

if __name__ == '__main__':
    app.run(port=8000, debug=True)



from twisted.internet import threads
from twisted.web import server
from twisted.web.client import getPage
from twisted.web.resource import Resource

from webservices.exceptions import BadRequest, WebserviceError
from webservices.models import BaseConsumer


class TwistedConsumer(BaseConsumer):
    def send_request(self, url, data, headers):
        return getPage(url, method='POST', postdata=data, headers=headers)

    def handle_response(self, response, max_age):
        def callback(body):
            return self.signer.loads(body, max_age=max_age)

        def errback(fail):
            message = fail.value.message
            status_code = fail.value.status
            self.raise_for_status(int(status_code), message)
        response.addCallback(callback)
        response.addErrback(errback)
        return response

    def raise_for_status(self, status_code, message):
        if status_code == 400:
            raise BadRequest(message)
        elif status_code >= 300:
            raise WebserviceError(message)


class ProviderResource(Resource):
    isLeaf = True

    def __init__(self, provider):
        self.provider = provider
        Resource.__init__(self)

    def render_POST(self, request):
        def get_header(key, default):
            return request.getHeader(key) or default

        def callback(info):
            status_code, data = info
            request.setResponseCode(status_code)
            request.write(data)
            request.finish()

        # XXX: This is not the correct way to pass the data to the processing
        #      function. The content is read from disk and could be too large
        #      to fit into memory. The correct way to go is to pass an open
        #      file handler as argument instead of a single string, but this
        #      would require each provider to be adapted.
        request.content.seek(0)
        signed_data = request.content.read()

        deferred = threads.deferToThread(
            self.provider.get_response,
            'POST',
            signed_data,
            get_header,
        )
        deferred.addCallback(callback)
        return server.NOT_DONE_YET


def provider_for_twisted(provider):
    return ProviderResource(provider)



from webservices.models import Provider

from api.models import Key


class HelloProvider(Provider):
    def get_private_key(self, public_key):
        try:
            return Key.objects.get(public_key=public_key).private_key
        except Key.DoesNotExist:
            return None

    def provide(self, data):
        name = data.get('name', 'world')
        return {'greeting': u'hello %s' % name}



from django.conf.urls import patterns, include, url
from webservices.sync import provider_for_django
from api.views import HelloProvider
from django.contrib import admin

admin.autodiscover()

urlpatterns = patterns(
    '',
    url(r'^$', provider_for_django(HelloProvider())),
    url(r'^admin/', include(admin.site.urls)),
)



# Django settings for django_app project.

TEMPLATE_DEBUG = DEBUG = True

MANAGERS = ADMINS = ()

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'keys.sqlite',
    }
}

TIME_ZONE = 'America/Chicago'

LANGUAGE_CODE = 'en-us'

SITE_ID = 1

USE_I18N = True

USE_L10N = True

USE_TZ = True

STATIC_URL = '/static/'

STATICFILES_DIRS = ()

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)

SECRET_KEY = 'j@)bmb^=#g4+$01mi^)jy)^bt!5zz+7%-^vqhc6d^39!eil804'

TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'django_app.urls'

WSGI_APPLICATION = 'django_app.wsgi.application'

TEMPLATE_DIRS = ()

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    'api',
)

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}



import argparse

from webservices.sync import SyncConsumer


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('public_key')
    parser.add_argument('private_key')
    parser.add_argument('name')
    parser.add_argument('--host', default='http://localhost:8000/')
    args = parser.parse_args()
    consumer = SyncConsumer(args.host, args.public_key, args.private_key)
    print(consumer.consume('/', {'name': args.name})['greeting'])

if __name__ == '__main__':
    main()



from twisted.internet import reactor
from twisted.web.server import Site

from webservices.async_helpers import provider_for_twisted
from webservices.models import Provider


API_KEYS = {
    'pubkey': 'privkey',  # your keys here
}


class HelloProvider(Provider):
    def get_private_key(self, public_key):
        return API_KEYS.get(public_key)

    def provide(self, data):
        name = data.get('name', 'world')
        return {'greeting': u'hello %s' % name}


resource = provider_for_twisted(HelloProvider())

site = Site(resource)
reactor.listenTCP(8000, site)
reactor.run()



import requests

from webservices.models import BaseConsumer


class SyncConsumer(BaseConsumer):
    def __init__(self, base_url, public_key, private_key):
        super(SyncConsumer, self).__init__(base_url, public_key, private_key)
        self.session = requests.session()

    def send_request(self, url, data, headers):  # pragma: no cover
        response = self.session.post(url, data=data, headers=headers)
        self.raise_for_status(response.status_code, response.content)
        return response.content


class DjangoTestingConsumer(SyncConsumer):
    def __init__(self, test_client, base_url, public_key, private_key):
        self.test_client = test_client
        super(DjangoTestingConsumer, self).__init__(
            base_url, public_key, private_key)

    def build_url(self, path):
        return path

    def send_request(self, url, data, headers):
        headers = {
            'HTTP_%s' % header.upper().replace('-', '_'): value
            for header, value in headers.items()
        }
        response = self.test_client.post(
            url,
            data=data,
            content_type='application/json',
            **headers
        )
        self.raise_for_status(response.status_code, response.content)
        return response.content


class FlaskTestingConsumer(DjangoTestingConsumer):
    def send_request(self, url, data, headers):
        response = self.test_client.post(url, data=data, headers=headers)
        self.raise_for_status(response.status_code, response.data)
        return response.data


def provider_for_django(provider):
    from django.http import HttpResponse
    from django.views.decorators.csrf import csrf_exempt

    def provider_view(request):
        def get_header(key, default):
            django_key = 'HTTP_%s' % key.upper().replace('-', '_')
            return request.META.get(django_key, default)
        method = request.method
        if getattr(request, 'body', None):
            signed_data = request.body
        else:
            signed_data = request.raw_post_data
        status_code, data = provider.get_response(
            method,
            signed_data,
            get_header,
        )
        return HttpResponse(data, status=status_code)
    return csrf_exempt(provider_view)


def provider_for_flask(app, url, provider):
    from flask import request

    def provider_view():
        def get_header(key, default):
            return request.headers.get(key, default)
        method = request.method
        signed_data = request.data
        status_code, data = provider.get_response(
            method,
            signed_data,
            get_header,
        )
        return data, status_code
    return app.route(url, methods=['POST'])(provider_view)



# must be first:
from webservices.sync import (
    provider_for_flask,
    FlaskTestingConsumer,
    provider_for_django,
    DjangoTestingConsumer,
)
from webservices.models import Provider, BaseConsumer, _split_dsn
from webservices.exceptions import BadRequest, WebserviceError
from webservices.async_helpers import provider_for_twisted, TwistedConsumer
from twisted.web.server import Site
from twisted.trial.unittest import TestCase as TwistedTestCase
from twisted.internet import reactor
from django.test.testcases import TestCase as DjangoTestCase
from unittest import TestCase
import sys
from django.conf import settings
settings.configure(
    ROOT_URLCONF='webservices.tests',
    ALLOWED_HOSTS='*',
    DATABASES={
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': ':memory:'
        },
    },
)

# real import


urlpatterns = []


class GreetingProvider(Provider):
    keys = {
        'pubkey': 'privatekey'
    }

    def __init__(self):
        self.exceptions = []

    def get_private_key(self, key):
        return self.keys.get(key)

    def report_exception(self):
        self.exceptions.append(sys.exc_info())

    def provide(self, data):
        if data.get('error'):
            raise Exception('Error')
        name = data.get('name', 'World')
        return {'greeting': u'Hello %s!' % name}


class GetFlaskTestingConsumer(FlaskTestingConsumer):
    def send_request(self, url, data, headers):  # pragma: no cover
        response = self.test_client.get(url, data=data, headers=headers)
        self.raise_for_status(response.status_code, response.data)
        return response.data


class BaseTests(TestCase):
    def test_consume_base_consumer(self):
        consumer = BaseConsumer('http://localhost', 'pubkey', 'privatekey')
        self.assertRaises(NotImplementedError,
                          consumer.consume, '/', {'name': 'Test'})

    def test_split_dsn(self):
        dsn = 'https://pub:priv@hostname.tld:1234/path'
        base_url, public_key, private_key = _split_dsn(dsn)
        self.assertEqual(base_url, 'https://hostname.tld:1234/path')
        self.assertEqual(public_key, 'pub')
        self.assertEqual(private_key, 'priv')


class FlaskTests(TestCase):
    def setUp(self):
        from flask import Flask
        app = Flask(__name__)
        app.config['TESTING'] = True
        self.provider = GreetingProvider()
        provider_for_flask(app, '/', self.provider)
        self.client = app.test_client()

    def test_greeting_provider(self):
        consumer = FlaskTestingConsumer(
            self.client, 'http://localhost', 'pubkey', 'privatekey')
        output = consumer.consume('/', {'name': 'Test'})
        self.assertEqual(output['greeting'], 'Hello Test!')

    def test_greeting_provider_wrong_key(self):
        consumer = FlaskTestingConsumer(
            self.client, 'http://localhost', 'pubkey', 'wrongkey')
        self.assertRaises(BadRequest, consumer.consume, '/', {'name': 'Test'})

    def test_starts_with_slash(self):
        consumer = FlaskTestingConsumer(
            self.client, 'http://localhost', 'pubkey', 'wrongkey')
        self.assertRaises(ValueError,
                          consumer.consume, 'wrong', {'name': 'Test'})

    def test_method_not_allowed(self):
        consumer = GetFlaskTestingConsumer(
            self.client, 'http://localhost', 'pubkey', 'wrongkey')
        self.assertRaises(WebserviceError,
                          consumer.consume, '/', {'name': 'Test'})

    def test_exception_hook(self):
        consumer = FlaskTestingConsumer(
            self.client, 'http://localhost', 'pubkey', 'privatekey')
        self.assertRaises(BadRequest, consumer.consume, '/', {'error': True})
        self.assertEqual(len(self.provider.exceptions), 1)


class DjangoTests(DjangoTestCase):
    def setUp(self):
        from django.test.client import Client
        self.client = Client()

    @property
    def urls(self):
        from django.conf.urls import url, patterns
        return patterns(
            '',
            url(r'^$', provider_for_django(GreetingProvider())),
        )

    def test_greeting_provider(self):
        consumer = DjangoTestingConsumer(
            self.client, 'http://localhost', 'pubkey', 'privatekey')
        output = consumer.consume('/', {'name': 'Test'})
        self.assertEqual(output['greeting'], 'Hello Test!')

    def test_greeting_provider_wrong_key(self):
        consumer = DjangoTestingConsumer(
            self.client, 'http://localhost', 'pubkey', 'wrongkey')
        self.assertRaises(BadRequest, consumer.consume, '/', {'name': 'Test'})


class TwistedTests(TwistedTestCase):
    def setUp(self):
        resource = provider_for_twisted(GreetingProvider())
        factory = Site(resource)
        self.port = reactor.listenTCP(0, factory, interface="127.0.0.1")
        self.client = None

    def tearDown(self):
        if self.client is not None:
            self.client.transport.loseConnection()
        return self.port.stopListening()

    def _test(self, public_key, private_key, path, data):
        base_url = 'http://127.0.0.1:%s/' % self.port.getHost().port
        consumer = TwistedConsumer(base_url, public_key, private_key)
        return consumer.consume(path, data)

    def test_greeting_provider(self):
        def cb(result):
            self.assertEqual(result['greeting'], 'Hello Test!')
        d = self._test('pubkey', 'privatekey', '/', {'name': 'Test'})
        d.addCallback(cb)
        return d

    def test_greeting_provider_wrong_key(self):
        def cb(result):
            self.assertRaises(BadRequest, result.raiseException)
        d = self._test('pubkey', 'wrongkey', '/', {'name': 'Test'})
        d.addErrback(cb)
        return d



from django.db import models


class Key(models.Model):
    public_key = models.CharField(max_length=100, unique=True)
    private_key = models.CharField(max_length=100, unique=True)

    def __unicode__(self):
        return 'Public Key: %s, Private Key: %s' % (
            self.public_key,
            self.private_key,
        )
